@@ -1,7 +1,8 @@
+
 /*
  * Copyright © 2009 CNRS
- * Copyright © 2009-2011 inria.  All rights reserved.
- * Copyright © 2009 Université Bordeaux 1
+ * Copyright © 2009-2015 Inria.  All rights reserved.
+ * Copyright © 2009 Université Bordeaux
  * Copyright © 2011 Cisco Systems, Inc.  All rights reserved.
  * See COPYING in top-level directory.
  */
@@ -15,24 +16,37 @@
 
 /* check hwloc_bitmap_asprintf(), hwloc_obj_cpuset_snprintf() and hwloc_bitmap_sscanf() */
 
-static void check_cpuset(hwloc_bitmap_t set, const char *expected)
+static void check_cpuset(hwloc_bitmap_t set, const char *expected1, const char *expected2, const char *expected3)
 {
   hwloc_bitmap_t set2 = hwloc_bitmap_alloc();
   char *string = NULL;
 
   hwloc_bitmap_asprintf(&string, set);
-  if (expected && strcmp(string, expected))
+  fprintf(stderr, "exported to %s\n", string);
+  if (expected1 && strcmp(string, expected1)) {
+    fprintf(stderr, "got %s instead of %s\n", string, expected1);
     assert(0);
+  }
   hwloc_bitmap_sscanf(set2, string);
   free(string);
   assert(hwloc_bitmap_isequal(set, set2));
 
   hwloc_bitmap_list_asprintf(&string, set);
+  fprintf(stderr, "exported to list %s\n", string);
+  if (expected2 && strcmp(string, expected2)) {
+    fprintf(stderr, "got %s instead of %s\n", string, expected2);
+    assert(0);
+  }
   hwloc_bitmap_list_sscanf(set2, string);
   free(string);
   assert(hwloc_bitmap_isequal(set, set2));
 
   hwloc_bitmap_taskset_asprintf(&string, set);
+  fprintf(stderr, "exported to taskset %s\n", string);
+  if (expected3 && strcmp(string, expected3)) {
+    fprintf(stderr, "got %s instead of %s\n", string, expected3);
+    assert(0);
+  }
   hwloc_bitmap_taskset_sscanf(set2, string);
   free(string);
   assert(hwloc_bitmap_isequal(set, set2));
@@ -51,21 +65,41 @@ int main(void)
 
   /* check an empty cpuset */
   set = hwloc_bitmap_alloc();
-  check_cpuset(set, "0x0");
-  hwloc_bitmap_free(set);
+  check_cpuset(set, "0x0", "", "0x0");
   printf("empty cpuset converted back and forth, ok\n");
 
+  /* make sure the first ulong is allocated even if empty */
+  hwloc_bitmap_set_ith_ulong(set, 0, 0UL);
+  check_cpuset(set, "0x0", "", "0x0");
+  printf("uselessly modified empty cpuset converted back and forth, ok\n");
+
+  /* make sure the 6th ulong is allocated even if empty */
+  hwloc_bitmap_set_ith_ulong(set, 5, 0UL);
+  check_cpuset(set, "0x0", "", "0x0");
+  printf("twice uselessly modified empty cpuset converted back and forth, ok\n");
+  hwloc_bitmap_free(set);
+
   /* check a full (and infinite) cpuset */
   set = hwloc_bitmap_alloc_full();
-  check_cpuset(set, "0xf...f");
-  hwloc_bitmap_free(set);
+  check_cpuset(set, "0xf...f", "0-", "0xf...f");
   printf("full cpuset converted back and forth, ok\n");
 
+  /* make sure the first ulong is allocated even if full */
+  hwloc_bitmap_set_ith_ulong(set, 0, ~0UL);
+  check_cpuset(set, "0xf...f", "0-", "0xf...f");
+  printf("uselessly modified full cpuset converted back and forth, ok\n");
+
+  /* make sure the 9th ulong is allocated even if full */
+  hwloc_bitmap_set_ith_ulong(set, 8, ~0UL);
+  check_cpuset(set, "0xf...f", "0-", "0xf...f");
+  printf("twice uselessly modified full cpuset converted back and forth, ok\n");
+  hwloc_bitmap_free(set);
+
   /* check an infinite (but non full) cpuset */
   set = hwloc_bitmap_alloc_full();
   hwloc_bitmap_clr(set, 173);
   hwloc_bitmap_clr_range(set, 60, 70);
-  check_cpuset(set, NULL);
+  check_cpuset(set, "0xf...f,0xffffdfff,0xffffffff,0xffffffff,0xffffff80,0x0fffffff,0xffffffff", "0-59,71-172,174-", "0xf...fffffdfffffffffffffffffffffffff800fffffffffffffff");
   hwloc_bitmap_free(set);
   printf("infinite/nonfull cpuset converted back and forth, ok\n");
 
@@ -73,7 +107,7 @@ int main(void)
   set = hwloc_bitmap_alloc();
   hwloc_bitmap_set(set, 2);
   hwloc_bitmap_set_range(set, 67, 70);
-  check_cpuset(set, "0x00000078,,0x00000004");
+  check_cpuset(set, "0x00000078,,0x00000004", "2,67-70", "0x780000000000000004");
   hwloc_bitmap_free(set);
   printf("finite/nonnull cpuset converted back and forth, ok\n");
 
@@ -85,7 +119,7 @@ int main(void)
   obj = hwloc_get_root_obj(topology);
   stringlen = hwloc_bitmap_asprintf(&string, obj->cpuset);
   printf("system cpuset is %s\n", string);
-  check_cpuset(obj->cpuset, NULL);
+  check_cpuset(obj->cpuset, NULL, NULL, NULL);
   printf("system cpuset converted back and forth, ok\n");
 
   printf("truncating system cpuset to NULL buffer\n");
@@ -152,13 +186,13 @@ int main(void)
   obj = hwloc_get_obj_by_depth(topology, depth-1, 0);
   hwloc_obj_cpuset_snprintf(string, stringlen+1, 1, &obj);
   printf("first cpu cpuset is %s\n", string);
-  check_cpuset(obj->cpuset, NULL);
+  check_cpuset(obj->cpuset, NULL, NULL, NULL);
   printf("first cpu cpuset converted back and forth, ok\n");
 
   obj = hwloc_get_obj_by_depth(topology, depth-1, hwloc_get_nbobjs_by_depth(topology, depth-1) - 1);
   hwloc_obj_cpuset_snprintf(string, stringlen+1, 1, &obj);
   printf("last cpu cpuset is %s\n", string);
-  check_cpuset(obj->cpuset, NULL);
+  check_cpuset(obj->cpuset, NULL, NULL, NULL);
   printf("last cpu cpuset converted back and forth, ok\n");
 
   free(string);
